package com.perforce.hws.automation;

import com.perforce.hws.p4base.CloseableServer;
import com.perforce.hws.p4base.CloseableServerProvider;
import com.perforce.hws.p4base.P4JavaRuntimeException;
import com.perforce.p4java.exception.*;
import com.perforce.p4java.impl.mapbased.rpc.RpcServer;
import com.perforce.p4java.option.server.TrustOptions;
import com.perforce.p4java.server.IServer;
import com.perforce.p4java.server.ServerFactory;

import java.net.URISyntaxException;
import java.util.Properties;

/**
 * Generates *very* basic server connections to the test instance running
 * locally.
 */
public interface AutomationServerProvider extends UsesAutomationConfig, CloseableServerProvider {

    @Override
    default CloseableServer obtain(String id, String user, String ticket) {
        if (id != null && !"default".equals(id)) {
            throw new IllegalArgumentException("default connection only for automation");
        }
        try {
            String uri = config().getP4dServerUri();
            Properties p = config().getP4javaProps();

            IServer s = ServerFactory.getServer(uri, p);

            if (config().isEnableP4dTls()) {
                RpcServer rpcServer = (RpcServer)s;
                rpcServer.addTrust(new TrustOptions(true, false, true));
            }

            if (config().isEnableUnicodeP4d()) {
                s.setCharsetName("utf8");
            }

            if (user != null) {
                s.setUserName(user);
            }

            s.connect();

            if (ticket != null) {
                s.setAuthTicket(ticket);
            }

            return new CloseableServer(this, s);
        } catch (URISyntaxException | P4JavaException e) {
            throw new P4JavaRuntimeException(e);
        }
    }
}
