package com.perforce.hws.automation;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardCopyOption;

/**
 * Created by tristan on 4/25/16.
 */
public class CopyCoverage implements ProjectLocations {

    private AutomationConfig config;

    private static final Logger logger = LoggerFactory.getLogger(CopyCoverage.class);

    private String targetPath;

    public CopyCoverage(String targetPath) {
        this.config = new AutomationConfig();
        this.targetPath = targetPath;
    }

    public AutomationConfig getConfig() {
        return config;
    }

    public void setConfig(AutomationConfig config) {
        this.config = config;
    }

    public AutomationConfig config() {
        return config;
    }

    public void run() throws Exception {
        Path sourcePath = Paths.get(config().getInstallDir(), "jacoco.exec");
        logger.debug("looking for coverage file at {}", sourcePath);
        if (Files.exists(sourcePath)) {
            Path targetPath = Paths.get(this.targetPath);

            if (!Files.exists(targetPath.getParent())) {
                logger.info("Creating coverage output dir {}", targetPath.getParent());
                Files.createDirectories(targetPath.getParent());
            }

            logger.info("Copying coverage file from {} to {}", sourcePath, targetPath);
            Files.copy(sourcePath, targetPath, StandardCopyOption.REPLACE_EXISTING);
        }
    }

    public static void main(String args[]) {
        try {
            new CopyCoverage(args[0]).run();
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(1);
        }
    }
}
