package com.perforce.hws.automation;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perforce.hws.util.FilesUtils;
import com.perforce.hws.util.ProcessHelpers;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;

/**
 *
 */
public interface FilesystemMethods extends UsesAutomationConfig, FilesUtils, OsHelpers, ProcessHelpers {

    Logger logger = LoggerFactory.getLogger(FilesystemMethods.class);

    /**
     * Completely rebuilds our temporary installation directory.
     * <br/>
     * We expect anything that was running, and had file handles open here,
     * should have been shut down previously. That's up to the caller.
     */
    default void deleteInstallationDir() throws IOException {
        File dir = new File(config().getInstallDir());
        if (dir.exists()) {
            System.out.println("Deleting " + config().getInstallDir());
            rmtree(config().getInstallDir());
        }
    }


    default void createInstallationDir() throws IOException {
        File dir = new File(config().getInstallDir());
        if (!dir.exists() && !dir.mkdirs()) {
            throw new IllegalStateException("Could not create dir: " +
                    config().getInstallDir());
        }
        Path logDir = Paths.get(config().getLogDir());
        if (!Files.exists(logDir)) {
            Files.createDirectories(logDir);
        }
        Path p4dRoot = Paths.get(config().getP4dRoot());
        if (!Files.exists(p4dRoot)) {
            Files.createDirectories(p4dRoot);
        }
    }

    default void dumpLog(Path logFile) {
        logger.info("reading file {}", logFile);
        try {
            Files.lines(logFile)
                    .forEach(line -> logger.debug(line));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
}
