package com.perforce.hws.automation;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.IOException;
import java.nio.file.Paths;
import java.util.stream.Collectors;

/**
 * These methods make it kind of trivial to launch the simulate script by
 * figuring out the directory paths for you.
 */
public interface SimulateECMethods extends ProjectLocations {

    Logger logger = LoggerFactory.getLogger(SimulateECMethods.class);

    default File codelineDir() throws IOException {
        return Paths.get(projectRoot(), "..", "..")
                .normalize().toAbsolutePath().toFile();
    }

    default String ecSimulateBuildPath() throws IOException {
        return Paths.get(projectRootOrFail(), "..", "..", "tools", "build",
                "test", "ec-simulate-platform-build.pl")
                .normalize().toAbsolutePath().toString();
    }

    default String codeline() {
        return "main";
    }

    default void launch(String job, String platform) throws IOException, InterruptedException {
        ProcessBuilder pb = new ProcessBuilder();
        pb.directory(codelineDir());
        pb.inheritIO();
        pb.command(ecSimulateBuildPath(),
                "-m", job,
                "-b", codeline(),
                "-p", platform,
                "--no_sync");
        Process p = pb.start();
        p.waitFor();
        if (p.exitValue() != 0) {
            throw new IllegalStateException("Failed with code " + p.exitValue());
        }
    }

    default void launchWindows(String job, String platform) throws IOException, InterruptedException {
        ProcessBuilder pb = new ProcessBuilder();
        pb.directory(codelineDir());
        pb.inheritIO();
        pb.command("C:\\Perl64\\bin\\perl.exe",
                ecSimulateBuildPath(),
                "-m", job,
                "-b", codeline(),
                "-p", platform,
                "--no_sync");
        logger.info("executing:\n  {}", pb.command().stream().collect(Collectors.joining(" ")));
        Process p = pb.start();
        p.waitFor();
        if (p.exitValue() != 0) {
            throw new IllegalStateException("Failed with code " + p.exitValue());
        }
    }
}
