package com.perforce.hws.automation;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perforce.hws.util.ProcessHelpers;

import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;


/**
 * This will install a deb package (from a known location within the project tree), then will
 * run an upgrade, which has been unpacked from a tar file.
 *
 * Both calls will have
 */
public class UpgradeDebPackage implements ProcessHelpers, OsHelpers {

    static Logger logger = LoggerFactory.getLogger(UpgradeDebPackage.class);

    public static void main(String args[]) {
//        System.err.println("config: " + System.getProperty("log4j.configurationFile"));

        try {
            new UpgradeDebPackage(args[0], args[1]).run();
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(1);
        }
    }

    private String baseDebPath;
    private String updateTarPath;

    public UpgradeDebPackage(String baseDebPath, String updateTarPath) {
        this.baseDebPath = baseDebPath;
        this.updateTarPath = updateTarPath;
    }

    public void run() throws Exception {

        logger.info("installing base package from {}", baseDebPath);

        execOrFail("dpkg", "-i", baseDebPath);

        Files.createDirectories(Paths.get("upgrade"));

        if (isVMWareFolder()) {
            execOrFail("sudo", "-u", vmwareUser(), "tar", "-x", "-z", "-p", "-C", "upgrade", "-f", updateTarPath);
        } else {
            execOrFail("tar", "-x", "-z", "-C", "upgrade", "-f", updateTarPath);
        }

        Path pkg = Files.newDirectoryStream(Paths.get("upgrade"), "helix-web-services*.deb").iterator().next();
        logger.info("Upgrading from {}", pkg);

        execOrFail("dpkg", "-i", pkg.toAbsolutePath().toString());
        execOrFail("service", "helix-ws", "restart");
    }


    public boolean isVMWareFolder() {
        return System.getenv("IS_VMWARE_FOLDER") != null;
    }

    public String vmwareUser() {
        return System.getenv("VMWARE_USER");
    }
}
